@@ -3,6 +3,8 @@
 __all__ = [
     'absorption_probabilities',
     'committor_probabilities',
+    'expected_rewards',
+    'expected_transitions',
     'first_passage_probabilities',
     'first_passage_reward',
     'hitting_probabilities',
@@ -13,7 +15,9 @@ __all__ = [
     'mean_number_visits',
     'mean_recurrence_times',
     'mixing_time',
-    'sensitivity'
+    'sensitivity',
+    'time_correlations',
+    'time_relaxations'
 ]
 
 
@@ -38,30 +42,16 @@ from .custom_types import *
 
 def absorption_probabilities(mc: tmc) -> oarray:
 
-    if len(mc.transient_states) == 0:
+    if not mc.is_absorbing or len(mc.transient_states) == 0:
         return None
 
     n = mc.fundamental_matrix
 
-    if mc.is_absorbing:
+    absorbing_indices = [mc.states.index(state) for state in mc.absorbing_states]
+    transient_indices = [mc.states.index(state) for state in mc.transient_states]
+    r = mc.p[np.ix_(transient_indices, absorbing_indices)]
 
-        absorbing_indices = [mc.states.index(state) for state in mc.absorbing_states]
-        transient_indices = [mc.states.index(state) for state in mc.transient_states]
-        r = mc.p[np.ix_(transient_indices, absorbing_indices)]
-
-        ap = np.transpose(np.matmul(n, r))
-
-    else:
-
-        recurrent_indices = [[*map(mc.states.index, recurrent_class)] for recurrent_class in mc.recurrent_classes]
-        transient_indices = [mc.states.index(state) for state in mc.transient_states]
-        r = np.zeros((len(transient_indices), len(recurrent_indices)), dtype=float)
-
-        for i, transient_state in enumerate(transient_indices):
-            for j, recurrent_class in enumerate(recurrent_indices):
-                r[i, j] = np.sum(mc.p[transient_state, :][:, recurrent_class])
-
-        ap = np.transpose(np.matmul(n, r))
+    ap = np.transpose(np.matmul(n, r))
 
     return ap
 
@@ -94,6 +84,51 @@ def committor_probabilities(mc: tmc, committor_type: str, states1: tlist_int, st
     return cp
 
 
+def expected_rewards(p: tarray, steps: int, rewards: tarray) -> tany:
+
+    original_rewards = np.copy(rewards)
+
+    er = np.copy(rewards)
+
+    for _ in range(steps):
+        er = original_rewards + np.dot(er, p)
+
+    return er
+
+
+def expected_transitions(p: tarray, rdl: trdl, steps: int, initial_distribution: tarray) -> tarray:
+
+    if steps <= p.shape[0]:
+
+        idist = initial_distribution
+        idist_sum = initial_distribution
+
+        for _ in range(steps - 1):
+            pi = np.dot(idist, p)
+            idist_sum += pi
+
+        et = idist_sum[:, np.newaxis] * p
+
+    else:
+
+        r, d, l = rdl  # noqa
+
+        q = np.asarray(np.diagonal(d))
+        q_indices = (q == 1.0)
+
+        gs = np.zeros(np.shape(q), dtype=float)
+        gs[q_indices] = steps
+        gs[~q_indices] = (1.0 - q[~q_indices]**steps) / (1.0 - q[~q_indices])
+
+        ds = np.diag(gs)
+        ts = np.dot(np.dot(r, ds), np.conjugate(l))
+        ps = np.dot(initial_distribution, ts)
+
+        et = np.real(ps[:, np.newaxis] * p)
+
+    return et
+
+
 def first_passage_probabilities(mc: tmc, steps: int, initial_state: int, first_passage_states: olist_int) -> tarray:
 
     e = np.ones((mc.size, mc.size), dtype=float) - np.eye(mc.size, dtype=float)
@@ -202,10 +237,10 @@ def hitting_times(mc: tmc, targets: tlist_int) -> tarray:
 def mean_absorption_times(mc: tmc) -> oarray:
 
     if not mc.is_absorbing or len(mc.transient_states) == 0:
-        mat = None
-    else:
-        n = mc.fundamental_matrix
-        mat = np.transpose(np.dot(n, np.ones(n.shape[0], dtype=float)))
+        return None
+
+    n = mc.fundamental_matrix
+    mat = np.transpose(np.dot(n, np.ones(n.shape[0], dtype=float)))
 
     return mat
 
@@ -215,17 +250,10 @@ def mean_first_passage_times_between(mc: tmc, origins: tlist_int, targets: tlist
     if not mc.is_ergodic:
         return None
 
-    a = np.eye(mc.size, dtype=float) - mc.p
-    a[targets, :] = 0.0
-    a[targets, targets] = 1.0
+    mfptt = mean_first_passage_times_to(mc, targets)
 
-    b = np.ones(mc.size, dtype=float)
-    b[targets] = 0.0
-
-    mfptt = npl.solve(a, b)
-
-    pi_origin_states = mc.pi[0][origins]
-    mu = pi_origin_states / np.sum(pi_origin_states)
+    pi_origins = mc.pi[0][origins]
+    mu = pi_origins / np.sum(pi_origins)
 
     mfptb = np.dot(mu, mfptt[origins])
 
@@ -342,9 +370,9 @@ def mean_number_visits(mc: tmc) -> oarray:
 def mean_recurrence_times(mc: tmc) -> oarray:
 
     if not mc.is_ergodic:
-        mrt = None
-    else:
-        mrt = np.array([0.0 if np.isclose(v, 0.0) else 1.0 / v for v in mc.pi[0]])
+        return None
+
+    mrt = np.array([0.0 if np.isclose(v, 0.0) else 1.0 / v for v in mc.pi[0]])
 
     return mrt
 
@@ -390,3 +418,167 @@ def sensitivity(mc: tmc, state: int) -> oarray:
     s = -np.outer(rev, phi) + (np.dot(phi, rev) * np.outer(rev, lev))
 
     return s
+
+
+def time_correlations(mc: tmc, rdl: trdl, walk1: twalk, walk2: owalk, time_points: ttimes_in) -> otimes_out:
+
+    if len(mc.pi) > 1:
+        return None
+
+    pi = mc.pi[0]
+
+    observations1 = np.zeros(mc.size, dtype=float)
+
+    for state in walk1:
+        observations1[state] += 1.0
+
+    if walk2 is None:
+        observations2 = np.copy(observations1)
+    else:
+
+        observations2 = np.zeros(mc.size, dtype=int)
+
+        for state in walk2:
+            observations2[state] += 1.0
+
+    if isinstance(time_points, int):
+        time_points = [time_points]
+        time_points_integer = True
+        time_points_length = 1
+    else:
+        time_points_integer = False
+        time_points_length = len(time_points)
+
+    tcs = []
+
+    if time_points[-1] > mc.size:
+
+        r, d, l = rdl  # noqa
+
+        for i in range(time_points_length):
+
+            t = np.zeros(d.shape, dtype=float)
+            t[np.diag_indices_from(d)] = np.diag(d)**time_points[i]
+
+            p_times = np.dot(np.dot(r, t), l)
+
+            m1 = np.multiply(observations1, pi)
+            m2 = np.dot(p_times, observations2)
+
+            tcs.append(np.dot(m1, m2).item())
+
+    else:
+
+        start_values = None
+
+        m = np.multiply(observations1, pi)
+
+        for i in range(time_points_length):
+
+            time_point = time_points[i]
+
+            if start_values is not None:
+
+                pk_i = start_values[1]
+                time_prev = start_values[0]
+                t_diff = time_point - time_prev
+
+                for k in range(t_diff):
+                    pk_i = np.dot(mc.p, pk_i)
+
+            else:
+
+                if time_point >= 2:
+
+                    pk_i = np.dot(mc.p, np.dot(mc.p, observations2))
+
+                    for k in range(time_point - 2):
+                        pk_i = np.dot(mc.p, pk_i)
+
+                elif time_point == 1:
+                    pk_i = np.dot(mc.p, observations2)
+                else:
+                    pk_i = observations2
+
+            start_values = (time_point, pk_i)
+
+            tcs.append(np.dot(m, pk_i).item())
+
+    if time_points_integer:
+        return tcs[0]
+
+    return tcs
+
+
+def time_relaxations(mc: tmc, rdl: trdl, walk: twalk, initial_distribution: tarray, time_points: ttimes_in) -> otimes_out:
+
+    if len(mc.pi) > 1:
+        return None
+
+    observations = np.zeros(mc.size, dtype=float)
+
+    for state in walk:
+        observations[state] += 1.0
+
+    if isinstance(time_points, int):
+        time_points = [time_points]
+        time_points_integer = True
+        time_points_length = 1
+    else:
+        time_points_integer = False
+        time_points_length = len(time_points)
+
+    trs = []
+
+    if time_points[-1] > mc.size:
+
+        r, d, l = rdl  # noqa
+
+        for i in range(time_points_length):
+
+            t = np.zeros(d.shape, dtype=float)
+            t[np.diag_indices_from(d)] = np.diag(d)**time_points[i]
+
+            p_times = np.dot(np.dot(r, t), l)
+
+            trs.append(np.dot(np.dot(initial_distribution, p_times), observations).item())
+
+    else:
+
+        start_values = None
+
+        for i in range(time_points_length):
+
+            time_point = time_points[i]
+
+            if start_values is not None:
+
+                pk_i = start_values[1]
+                time_prev = start_values[0]
+                t_diff = time_point - time_prev
+
+                for k in range(t_diff):
+                    pk_i = np.dot(pk_i, mc.p)
+
+            else:
+
+                if time_point >= 2:
+
+                    pk_i = np.dot(np.dot(initial_distribution, mc.p), mc.p)
+
+                    for k in range(time_point - 2):
+                        pk_i = np.dot(pk_i, mc.p)
+
+                elif time_point == 1:
+                    pk_i = np.dot(initial_distribution, mc.p)
+                else:
+                    pk_i = initial_distribution
+
+            start_values = (time_point, pk_i)
+
+            trs.append(np.dot(pk_i, observations).item())
+
+    if time_points_integer:
+        return trs[0]
+
+    return trs
