@@ -48,10 +48,10 @@ from .validation import *
 # CONSTANTS #
 #############
 
-color_black = '#000000'
-color_gray = '#E0E0E0'
-color_white = '#FFFFFF'
-colors = ['#80B1D3', '#FFED6F', '#B3DE69', '#BEBADA', '#FDB462', '#8DD3C7', '#FB8072', '#FCCDE5']
+_color_black = '#000000'
+_color_gray = '#E0E0E0'
+_color_white = '#FFFFFF'
+_colors = ['#80B1D3', '#FFED6F', '#B3DE69', '#BEBADA', '#FDB462', '#8DD3C7', '#FB8072', '#FCCDE5']
 
 
 #############
@@ -106,9 +106,9 @@ def plot_eigenvalues(mc: tmc, dpi: int = 100) -> oplot:
 
                 cs = np.linspace(-1.1, 1.1, 201)
                 x_spectral_gap, y_spectral_gap = np.meshgrid(cs, cs)
-                z_spectral_gap = x_spectral_gap**2 + y_spectral_gap**2
+                z_spectral_gap = x_spectral_gap**2.0 + y_spectral_gap**2.0
 
-                h = ax.contourf(x_spectral_gap, y_spectral_gap, z_spectral_gap, alpha=0.2, colors='r', levels=[mu ** 2.0, 1.0])
+                h = ax.contourf(x_spectral_gap, y_spectral_gap, z_spectral_gap, alpha=0.2, colors='r', levels=[mu**2.0, 1.0])
                 handles.append(pp.Rectangle((0.0, 0.0), 1.0, 1.0, fc=h.collections[0].get_facecolor()[0]))
                 labels.append('Spectral Gap')
 
@@ -177,16 +177,19 @@ def plot_graph(mc: tmc, nodes_color: bool = True, nodes_type: bool = True, edges
 
     def node_colors(count: int) -> tlist_str:
 
-        colors_limit = len(colors) - 1
+        colors_limit = len(_colors) - 1
         offset = 0
 
         clist = list()
 
         while count > 0:
-            clist.append(colors[offset])
+
+            clist.append(_colors[offset])
             offset += 1
-            if offset > colors_limit:
+
+            if offset > colors_limit:  # pragma: no cover
                 offset = 0
+
             count -= 1
 
         return clist
@@ -213,13 +216,13 @@ def plot_graph(mc: tmc, nodes_color: bool = True, nodes_type: bool = True, edges
         # noinspection PyBroadException
         try:
             call(['dot', '-V'], stdout=PIPE, stderr=PIPE)
-        except Exception:
+        except Exception:  # pragma: no cover
             extended_graph = False
             pass
 
         try:
-            import pydot as pyd
-        except ImportError:
+            import pydot as pyd  # noqa
+        except ImportError:  # pragma: no cover
             extended_graph = False
             pass
 
@@ -247,7 +250,7 @@ def plot_graph(mc: tmc, nodes_color: bool = True, nodes_type: bool = True, edges
                     node.set_shape('ellipse')
 
         if edges_color:
-            c = edge_colors(color_gray, color_black, 20)
+            c = edge_colors(_color_gray, _color_black, 20)
             for edge in g_pydot.get_edges():
                 probability = mc.transition_probability(edge.get_destination(), edge.get_source())
                 x = int(round(probability * 20.0)) - 1
@@ -257,10 +260,7 @@ def plot_graph(mc: tmc, nodes_color: bool = True, nodes_type: bool = True, edges
         if edges_value:
             for edge in g_pydot.get_edges():
                 probability = mc.transition_probability(edge.get_destination(), edge.get_source())
-                if probability.is_integer():
-                    edge.set_label(f' {probability:g}.0 ')
-                else:
-                    edge.set_label(f' {round(probability,2):g} ')
+                edge.set_label(f' {round(probability,2):g} ')
 
         buffer = BytesIO()
         buffer.write(g_pydot.create_png())
@@ -322,11 +322,7 @@ def plot_graph(mc: tmc, nodes_color: bool = True, nodes_type: bool = True, edges
 
             for edge in g.edges:
                 probability = mc.transition_probability(edge[1], edge[0])
-                if probability.is_integer():
-                    value = f' {probability:g}.0 '
-                else:
-                    value = f' {round(probability,2):g} '
-                edges_values[(edge[0], edge[1])] = value
+                edges_values[(edge[0], edge[1])] = f' {round(probability,2):g} '
 
             nx.draw_networkx_edge_labels(g, positions, ax=ax, edge_labels=edges_values, label_pos=0.7)
 
@@ -358,10 +354,7 @@ def plot_redistributions(mc: tmc, distributions: tdists_flex, initial_status: os
 
         mc = validate_markov_chain(mc)
         distributions = validate_distribution(distributions, mc.size)
-
-        if initial_status is not None:
-            initial_status = validate_status(initial_status, mc.states)
-
+        initial_status = None if initial_status is None else validate_status(initial_status, mc.states)
         plot_type = validate_enumerator(plot_type, ['heatmap', 'projection'])
         dpi = validate_dpi(dpi)
 
@@ -369,25 +362,26 @@ def plot_redistributions(mc: tmc, distributions: tdists_flex, initial_status: os
         raise generate_validation_error(e, trace()) from None
 
     if isinstance(distributions, int):
-        distributions = mc.redistribute(distributions, initial_status=initial_status, include_initial=True, output_last=False)
-    elif initial_status is not None and not np.array_equal(distributions[0], initial_status):
+        distributions = mc.redistribute(distributions, initial_status=initial_status, output_last=False)
+
+    if initial_status is not None and not np.array_equal(distributions[0], initial_status):  # pragma: no cover
         raise ValueError('The "initial_status" parameter, if specified when the "distributions" parameter represents a sequence of redistributions, must match the first element.')
 
-    distribution_len = len(distributions)
-    distributions = np.array(distributions)
+    distributions_len = 1 if isinstance(distributions, np.ndarray) else len(distributions)
+    distributions = np.array([distributions]) if isinstance(distributions, np.ndarray) else np.array(distributions)
 
     figure, ax = pp.subplots(dpi=dpi)
 
     if plot_type == 'heatmap':
 
-        color_map = mplc.LinearSegmentedColormap.from_list('ColorMap', [color_white, colors[0]], 20)
+        color_map = mplc.LinearSegmentedColormap.from_list('ColorMap', [_color_white, _colors[0]], 20)
         ax_is = ax.imshow(np.transpose(distributions), aspect='auto', cmap=color_map, interpolation='none', vmin=0.0, vmax=1.0)
 
         ax.set_xlabel('Steps', fontsize=13.0)
-        ax.set_xticks(np.arange(0.0, distribution_len + 1.0, 1.0 if distribution_len <= 11 else 10.0), minor=False)
-        ax.set_xticks(np.arange(-0.5, distribution_len, 1.0), minor=True)
-        ax.set_xticklabels(np.arange(0, distribution_len, 1 if distribution_len <= 11 else 10))
-        ax.set_xlim(-0.5, distribution_len - 0.5)
+        ax.set_xticks(np.arange(0.0, distributions_len + 1.0, 1.0 if distributions_len <= 11 else 10.0), minor=False)
+        ax.set_xticks(np.arange(-0.5, distributions_len, 1.0), minor=True)
+        ax.set_xticklabels(np.arange(0, distributions_len + 1, 1 if distributions_len <= 11 else 10))
+        ax.set_xlim(-0.5, distributions_len - 0.5)
 
         ax.set_yticks(np.arange(0.0, mc.size, 1.0), minor=False)
         ax.set_yticks(np.arange(-0.5, mc.size, 1.0), minor=True)
@@ -402,25 +396,26 @@ def plot_redistributions(mc: tmc, distributions: tdists_flex, initial_status: os
 
     else:
 
-        ax.set_prop_cycle('color', colors)
+        ax.set_prop_cycle('color', _colors)
 
-        if distribution_len == 2:
+        if distributions_len == 2:
             for i in range(mc.size):
-                ax.plot(np.arange(0.0, distribution_len, 1.0), distributions[:, i], label=mc.states[i], marker='o')
+                ax.plot(np.arange(0.0, distributions_len, 1.0), distributions[:, i], label=mc.states[i], marker='o')
         else:
             for i in range(mc.size):
-                ax.plot(np.arange(0.0, distribution_len, 1.0), distributions[:, i], label=mc.states[i])
+                ax.plot(np.arange(0.0, distributions_len, 1.0), distributions[:, i], label=mc.states[i])
 
-        if np.array_equal(distributions[0, :], np.ones(mc.size, dtype=float) / mc.size):
-            ax.plot(0.0, distributions[0, 0], color=color_black, label="Start", marker='o', markeredgecolor=color_black, markerfacecolor=color_black)
+        if np.allclose(distributions[0, :], np.ones(mc.size, dtype=float) / mc.size):
+            ax.plot(0.0, distributions[0, 0], color=_color_black, label="Start", marker='o', markeredgecolor=_color_black, markerfacecolor=_color_black)
             legend_size = mc.size + 1
-        else:
+        else:  # pragma: no cover
             legend_size = mc.size
 
         ax.set_xlabel('Steps', fontsize=13.0)
-        ax.set_xticks(np.arange(0.0, distribution_len, 1.0 if distribution_len <= 11 else 10.0))
-        ax.set_xticklabels(np.arange(0, distribution_len, 1 if distribution_len <= 11 else 10))
-        ax.set_xlim(-1.0 * (distribution_len * 0.05), distribution_len * 1.05)
+        ax.set_xticks(np.arange(0.0, distributions_len + 1.0, 1.0 if distributions_len <= 11 else 10.0), minor=False)
+        ax.set_xticks(np.arange(-0.5, distributions_len, 1.0), minor=True)
+        ax.set_xticklabels(np.arange(0, distributions_len + 1, 1 if distributions_len <= 11 else 10))
+        ax.set_xlim(-0.5, distributions_len - 0.5)
 
         ax.set_ylabel('Frequencies', fontsize=13.0)
         ax.set_yticks(np.linspace(0.0, 1.0, 11))
@@ -458,11 +453,8 @@ def plot_walk(mc: tmc, walk: twalk_flex, initial_state: ostate = None, plot_type
 
         mc = validate_markov_chain(mc)
 
-        if isinstance(walk, int):
-            if initial_state is None:
-                walk = validate_integer(walk, lower_limit=(2, False))
-            else:
-                walk = validate_integer(walk, lower_limit=(1, False))
+        if isinstance(walk, (int, np.integer)):
+            walk = validate_integer(walk, lower_limit=(1, False))
         else:
             walk = validate_states(walk, mc.states, 'walk', False)
 
@@ -476,8 +468,9 @@ def plot_walk(mc: tmc, walk: twalk_flex, initial_state: ostate = None, plot_type
         raise generate_validation_error(e, trace()) from None
 
     if isinstance(walk, int):
-        walk = mc.walk(walk, initial_state=initial_state, include_initial=True, output_indices=True)
-    elif initial_state is not None and (walk[0] != initial_state):
+        walk = mc.walk(walk, initial_state=initial_state, output_indices=True)
+
+    if initial_state is not None and (walk[0] != initial_state):  # pragma: no cover
         raise ValueError('The "initial_state" parameter, if specified when the "walk" parameter represents a sequence of states, must match the first element.')
 
     walk_len = len(walk)
@@ -493,7 +486,7 @@ def plot_walk(mc: tmc, walk: twalk_flex, initial_state: ostate = None, plot_type
 
         walk_histogram = np.sum(walk_histogram, axis=1) / np.sum(walk_histogram)
 
-        ax.bar(np.arange(0.0, mc.size, 1.0), walk_histogram, edgecolor=color_black, facecolor=colors[0])
+        ax.bar(np.arange(0.0, mc.size, 1.0), walk_histogram, edgecolor=_color_black, facecolor=_colors[0])
 
         ax.set_xlabel('States', fontsize=13.0)
         ax.set_xticks(np.arange(0.0, mc.size, 1.0))
@@ -512,7 +505,7 @@ def plot_walk(mc: tmc, walk: twalk_flex, initial_state: ostate = None, plot_type
         for i, s in enumerate(walk):
             walk_sequence[s, i] = 1.0
 
-        color_map = mplc.LinearSegmentedColormap.from_list('ColorMap', [color_white, colors[0]], 2)
+        color_map = mplc.LinearSegmentedColormap.from_list('ColorMap', [_color_white, _colors[0]], 2)
         ax.imshow(walk_sequence, aspect='auto', cmap=color_map, interpolation='none', vmin=0.0, vmax=1.0)
 
         ax.set_xlabel('Steps', fontsize=13.0)
@@ -537,9 +530,9 @@ def plot_walk(mc: tmc, walk: twalk_flex, initial_state: ostate = None, plot_type
         for i in range(1, walk_len):
             walk_transitions[walk[i - 1], walk[i]] += 1.0
 
-        walk_transitions = walk_transitions / np.sum(walk_transitions)
+        walk_transitions /= np.sum(walk_transitions)
 
-        color_map = mplc.LinearSegmentedColormap.from_list('ColorMap', [color_white, colors[0]], 20)
+        color_map = mplc.LinearSegmentedColormap.from_list('ColorMap', [_color_white, _colors[0]], 20)
         ax_is = ax.imshow(walk_transitions, aspect='auto', cmap=color_map, interpolation='none', vmin=0.0, vmax=1.0)
 
         ax.set_xticks(np.arange(0.0, mc.size, 1.0), minor=False)
