@@ -26,7 +26,7 @@ from pytest import (
 # TESTS #
 #########
 
-def test_attributes(p, is_absorbing, is_canonical, is_ergodic, is_reversible, is_symmetric):
+def test_attributes(p, is_absorbing, is_canonical, is_doubly_stochastic, is_ergodic, is_reversible, is_symmetric):
 
     mc = MarkovChain(p)
 
@@ -40,6 +40,11 @@ def test_attributes(p, is_absorbing, is_canonical, is_ergodic, is_reversible, is
 
     assert actual == expected
 
+    actual = mc.is_doubly_stochastic
+    expected = is_doubly_stochastic
+
+    assert actual == expected
+
     actual = mc.is_ergodic
     expected = is_ergodic
 
